@@ -1,4 +1,4 @@
-# Copyright (C) 2012 Nick Boultbee, Thomas Vogt
+# Copyright (C) 2012-13 Nick Boultbee, Thomas Vogt
 # Copyright (C) 2008 Andreas Bombe
 # Copyright (C) 2005  Michael Urman
 # Based on osd.py (C) 2005 Ton van den Heuvel, Joe Wreshnig
@@ -23,50 +23,51 @@ from math import pi
 
 from quodlibet import config, qltk, app
 from quodlibet.qltk.textedit import PatternEdit
+from quodlibet.formats import DUMMY_SONG
 from quodlibet import parse
 from quodlibet.plugins.events import EventPlugin
 from quodlibet.plugins import PluginConfigMixin
-from quodlibet.util.dprint import print_d
+from quodlibet.util.dprint import print_d, print_w
 
 
-def Label(text):
-    l = Gtk.Label(label=text, use_underline=True)
-    l.set_alignment(0.0, 0.5)
-    return l
+class ConfigLabel(Gtk.Label):
+    """Customised Label for configuration, tied to a widget"""
+
+    def __init__(self, text, widget):
+        super(Gtk.Label, self).__init__(label=text, use_underline=True)
+        self.set_mnemonic_widget(widget)
+        self.set_alignment(0.0, 0.5)
 
 
 class OSDWindow(Gtk.Window):
     __gsignals__ = {
-            'fade-finished': (GObject.SignalFlags.RUN_LAST, None, (bool,)),
-            }
+        'fade-finished': (GObject.SignalFlags.RUN_LAST, None, (bool,)),
+    }
 
     def __init__(self, conf, song):
         Gtk.Window.__init__(self, Gtk.WindowType.POPUP)
         self.set_type_hint(Gdk.WindowTypeHint.NOTIFICATION)
 
-        # for non-composite operation
-        self.background_pixbuf = None
-        self.titleinfo_surface = None
-
         screen = self.get_screen()
-        # FIXME: GIPORT
-        """cmap = screen.get_rgba_colormap()
-        if cmap is None:
-            cmap = screen.get_rgb_colormap()
-        self.set_colormap(cmap)"""
+        rgba = screen.get_rgba_visual()
+        if rgba is not None:
+            self.set_visual(rgba)
 
         self.conf = conf
         self.iteration_source = None
+        self.fading_in = False
+        self.fade_start_time = 0
 
         cover = song.find_cover()
         try:
             if cover is not None:
                 cover = GdkPixbuf.Pixbuf.new_from_file(cover.name)
         except GLib.GError, gerror:
-            print 'Error while loading cover image:', gerror.message
+            print_w('GDK error while loading cover image: %s'
+                    % gerror.message)
         except:
-            from traceback import print_exc
-            print_exc()
+            from traceback import format_exc
+            print_w("Error loading cover image: %s" % format_exc())
 
         # now calculate size of window
         mgeo = screen.get_monitor_geometry(conf.monitor)
@@ -74,19 +75,19 @@ class OSDWindow(Gtk.Window):
         textwidth = mgeo.width - 2 * (conf.border + conf.margin)
         if cover is not None:
             textwidth -= coverwidth + conf.border
-            coverheight = int(cover.get_height() * (float(coverwidth) /
-                cover.get_width()))
+            coverheight = int(cover.get_height() * float(coverwidth)
+                              / cover.get_width())
         else:
             coverheight = 0
         self.cover_pixbuf = cover
 
         layout = self.create_pango_layout('')
-        layout.set_alignment((Pango.Alignment.LEFT,
-                Pango.Alignment.CENTER, Pango.Alignment.RIGHT)[conf.align])
+        layout.set_alignment((Pango.Alignment.LEFT, Pango.Alignment.CENTER,
+                              Pango.Alignment.RIGHT)[conf.align])
         layout.set_spacing(Pango.SCALE * 7)
         layout.set_font_description(Pango.FontDescription(conf.font))
         try:
-            layout.set_markup(parse.XMLFromPattern(conf.string) % song)
+            layout.set_markup(parse.XMLFromMarkupPattern(conf.string) % song)
         except parse.error:
             layout.set_markup("")
         layout.set_width(Pango.SCALE * textwidth)
@@ -117,48 +118,46 @@ class OSDWindow(Gtk.Window):
         self.move(winx + mgeo.x, winy + mgeo.y)
 
     def do_draw(self, cr):
-        self.draw_title_info(cr)
-        return
-
-        # FIXME: GIPORT
         if self.is_composited():
-            # the simple case
             self.draw_title_info(cr)
-            return
-
-        # manual transparency rendering follows
-        back_pbuf = self.background_pixbuf
-        title_surface = self.titleinfo_surface
-        walloc = self.get_allocation()
-        wpos = self.get_position()
-
-        if back_pbuf is None and 0:
-            root = self.get_screen().get_root_window()
-            back_pbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB,
-                    False, 8,
-                    walloc.width, walloc.height)
-            back_pbuf.get_from_drawable(root, root.get_colormap(),
-                    wpos[0], wpos[1], 0, 0, walloc.width, walloc.height)
-            self.background_pixbuf = back_pbuf
-
-        if title_surface is None:
-            title_surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
-                walloc.width, walloc.height)
-            titlecr = Gdk.cairo_create(self.get_window())
-            titlecr.set_source_surface(title_surface)
-            self.draw_title_info(titlecr)
-
-        cr.set_operator(cairo.OPERATOR_SOURCE)
-        if back_pbuf is not None:
-            cr.set_source_pixbuf(back_pbuf, 0, 0)
         else:
-            cr.set_source_rgb(0.3, 0.3, 0.3)
-        cr.paint()
-        cr.set_operator(cairo.OPERATOR_OVER)
-        cr.set_source_surface(title_surface, 0, 0)
-        cr.paint_with_alpha(self.get_opacity())
+            # manual transparency rendering follows
+            walloc = self.get_allocation()
+            wpos = self.get_position()
 
-    def rounded_rectangle(self, cr, x, y, radius, width, height):
+            if not getattr(self, "_bg_sf", None):
+                # copy the root surface into a temp image surface
+                root_win = self.get_root_window()
+                bg_sf = cairo.ImageSurface(cairo.FORMAT_ARGB32,
+                                           walloc.width, walloc.height)
+                pb = Gdk.pixbuf_get_from_window(
+                    root_win, wpos[0], wpos[1], walloc.width, walloc.height)
+                bg_cr = cairo.Context(bg_sf)
+                Gdk.cairo_set_source_pixbuf(bg_cr, pb, 0, 0)
+                bg_cr.paint()
+                self._bg_sf = bg_sf
+
+            if not getattr(self, "_fg_sf", None):
+                # draw the window content in another temp surface
+                fg_sf = cairo.ImageSurface(cairo.FORMAT_ARGB32,
+                                           walloc.width, walloc.height)
+                fg_cr = cairo.Context(fg_sf)
+                fg_cr.set_source_surface(fg_sf)
+                self.draw_title_info(fg_cr)
+                self._fg_sf = fg_sf
+
+            # first draw the background so we have 'transparancy'
+            cr.set_operator(cairo.OPERATOR_SOURCE)
+            cr.set_source_surface(self._bg_sf)
+            cr.paint()
+
+            # then draw the window content with the right opacity
+            cr.set_operator(cairo.OPERATOR_OVER)
+            cr.set_source_surface(self._fg_sf)
+            cr.paint_with_alpha(self.get_opacity())
+
+    @staticmethod
+    def rounded_rectangle(cr, x, y, radius, width, height):
         cr.move_to(x + radius, y)
         cr.line_to(x + width - radius, y)
         cr.arc(x + width - radius, y + radius, radius,
@@ -176,17 +175,28 @@ class OSDWindow(Gtk.Window):
 
     def draw_conf_rect(self, cr, x, y, width, height, radius):
         if self.conf.corners != 0:
-                self.rounded_rectangle(cr, x, y, radius, width, height)
+            self.rounded_rectangle(cr, x, y, radius, width, height)
         else:
-                cr.rectangle(x, y, width, height)
+            cr.rectangle(x, y, width, height)
 
     def draw_title_info(self, cr):
+        cr.save()
         do_shadow = (self.conf.shadow[0] != -1.0)
         do_outline = (self.conf.outline[0] != -1.0)
 
-        cr.set_operator(cairo.OPERATOR_CLEAR)
-        cr.rectangle(0, 0, *self.get_size())
-        cr.fill()
+        self.set_name("osd_bubble")
+        style_provider = Gtk.CssProvider()
+        css = """
+            #osd_bubble {
+                background-color:rgba(0,0,0,0);
+            }
+        """
+        style_provider.load_from_data(css.encode("utf8"))
+        style_context = self.get_style_context()
+        style_context.add_provider(
+            style_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        )
 
         cr.set_operator(cairo.OPERATOR_OVER)
         cr.set_source_rgba(*self.conf.fill)
@@ -201,8 +211,10 @@ class OSDWindow(Gtk.Window):
             f = self.conf.fill
             rgba = (f[0] / 1.25, f[1] / 1.25, f[2] / 1.25, f[3] / 2.0)
             cr.set_source_rgba(*rgba)
-            self.draw_conf_rect(cr, 1, 1,
-                self.get_size()[0] - 2, self.get_size()[1] - 2, radius)
+            self.draw_conf_rect(cr,
+                                1, 1,
+                                self.get_size()[0] - 2, self.get_size()[1] - 2,
+                                radius)
             cr.set_line_width(2.0)
             cr.stroke()
 
@@ -216,25 +228,29 @@ class OSDWindow(Gtk.Window):
 
             if do_shadow:
                 cr.set_source_rgba(*self.conf.shadow)
-                self.draw_conf_rect(cr, rect.x + 2, rect.y + 2,
-                        rect.width, rect.height,
-                        0.6 * self.conf.corners * rect.width)
+                self.draw_conf_rect(cr,
+                                    rect.x + 2, rect.y + 2,
+                                    rect.width, rect.height,
+                                    0.6 * self.conf.corners * rect.width)
                 cr.fill()
 
             if do_outline:
                 cr.set_source_rgba(*self.conf.outline)
-                self.draw_conf_rect(cr, rect.x, rect.y,
-                        rect.width, rect.height,
-                        0.6 * self.conf.corners * rect.width)
+                self.draw_conf_rect(cr,
+                                    rect.x, rect.y,
+                                    rect.width, rect.height,
+                                    0.6 * self.conf.corners * rect.width)
                 cr.stroke()
 
             Gdk.cairo_set_source_pixbuf(cr, pbuf, 0, 0)
             transmat.scale(pbuf.get_width() / float(rect.width),
-                    pbuf.get_height() / float(rect.height))
+                           pbuf.get_height() / float(rect.height))
             transmat.translate(-rect.x, -rect.y)
             cr.get_source().set_matrix(transmat)
-            self.draw_conf_rect(cr, rect.x, rect.y,
-                 rect.width, rect.height, 0.6 * self.conf.corners * rect.width)
+            self.draw_conf_rect(cr,
+                                rect.x, rect.y,
+                                rect.width, rect.height,
+                                0.6 * self.conf.corners * rect.width)
             cr.fill()
 
         PangoCairo.update_layout(cr, self.title_layout)
@@ -253,6 +269,7 @@ class OSDWindow(Gtk.Window):
         cr.set_source_rgb(*self.conf.text[:3])
         cr.move_to(textx, texty)
         PangoCairo.show_layout(cr, self.title_layout)
+        cr.restore()
 
     def fade_in(self):
         self.do_fade_inout(True)
@@ -262,7 +279,6 @@ class OSDWindow(Gtk.Window):
 
     def do_fade_inout(self, fadein):
         fadein = bool(fadein)
-
         self.fading_in = fadein
         now = GObject.get_current_time()
 
@@ -298,7 +314,7 @@ class AnimOsd(EventPlugin, PluginConfigMixin):
     PLUGIN_ID = "Animated On-Screen Display"
     PLUGIN_NAME = _("Animated On-Screen Display")
     PLUGIN_DESC = _("Display song information on your screen when it changes.")
-    PLUGIN_VERSION = "1.2"
+    PLUGIN_VERSION = "1.3"
     # Retain compatibility with old configuration
     CONFIG_SECTION = 'animosd'
 
@@ -310,227 +326,241 @@ class AnimOsd(EventPlugin, PluginConfigMixin):
             return int(x * 65535)
 
         def cfg_set_tuple(name, t):
-            string = " ".join(map(str, t))
-            #print_d("Writing config: %s=%s" % (name, string))
-            self.config_set("%s" % name, string)
+            s = " ".join(map(str, t))
+            self.config_set("%s" % name, s)
 
-        class ConfigLabel(Gtk.Label):
-            def __init__(self, text, widget):
-                super(ConfigLabel, self).__init__(text)
-                self.set_use_underline(True)
-                self.set_mnemonic_widget(widget)
+        def show_preview():
+            preview_song = (app.player.song if app.player.song else DUMMY_SONG)
+            self.plugin_on_song_started(preview_song)
+
+        def on_button_pressed(x=None, y=None):
+            show_preview()
 
         def set_text(button):
             color = button.get_color()
             color = map(__coltofloat,
                         (color.red, color.green, color.blue, 0.0))
-            self.conf.text = tuple(color)
-            cfg_set_tuple("text", self.conf.text)
-            self.plugin_single_song(app.player.song)
+            self.Conf.text = tuple(color)
+            cfg_set_tuple("text", self.Conf.text)
+            show_preview()
 
         def set_fill(button):
             color = button.get_color()
             color = map(__coltofloat, (color.red, color.green, color.blue,
                 button.get_alpha()))
-            self.conf.fill = tuple(color)
-            cfg_set_tuple("fill", self.conf.fill)
-            self.plugin_single_song(app.player.song)
+            self.Conf.fill = tuple(color)
+            cfg_set_tuple("fill", self.Conf.fill)
+            show_preview()
 
         def set_font(button):
             font = button.get_font_name()
             self.config_set("font", font)
-            self.conf.font = font
-            self.plugin_single_song(app.player.song)
+            self.Conf.font = font
+            show_preview()
 
         def change_delay(button):
             value = int(button.get_value() * 1000)
             self.config_set("delay", str(value))
-            self.conf.delay = value
+            self.Conf.delay = value
 
         def change_monitor(button):
             """Monitor number config change handler"""
             value = int(button.get_value())
             self.config_set("monitor", str(value))
-            self.conf.monitor = value
-            self.plugin_single_song(app.player.song)
+            self.Conf.monitor = value
+            show_preview()
 
         def change_position(button):
             value = button.get_active() / 2.0
             self.config_set("pos_y", str(value))
-            self.conf.pos_y = value
-            self.plugin_single_song(app.player.song)
+            self.Conf.pos_y = value
+            show_preview()
 
         def change_align(button):
             value = button.get_active()
             self.config_set("align", str(value))
-            self.conf.align = value
-            self.plugin_single_song(app.player.song)
+            self.Conf.align = value
+            show_preview()
 
         def change_shadow(button):
             if button.get_active():
-                self.conf.shadow = (0.0, 0.0, 0.0, self.conf.fill[3])
+                self.Conf.shadow = (0.0, 0.0, 0.0, self.Conf.fill[3])
             else:
-                self.conf.shadow = (-1.0, 0.0, 0.0, 0.0)
-            cfg_set_tuple("shadow", self.conf.shadow)
-            self.plugin_single_song(app.player.song)
+                self.Conf.shadow = (-1.0, 0.0, 0.0, 0.0)
+            cfg_set_tuple("shadow", self.Conf.shadow)
+            show_preview()
 
         def change_outline(button):
             if button.get_active():
                 # Vary with fill alpha to create a smoother outline edge
-                alpha = (min(1.0, self.conf.fill[3] * 1.25))
-                self.conf.outline = (0.1, 0.1, 0.1, alpha)
+                alpha = (min(1.0, self.Conf.fill[3] * 1.25))
+                self.Conf.outline = (0.1, 0.1, 0.1, alpha)
             else:
-                self.conf.outline = (-1.0, 0.0, 0.0)
-            cfg_set_tuple("outline", self.conf.outline)
-            self.plugin_single_song(app.player.song)
+                self.Conf.outline = (-1.0, 0.0, 0.0)
+            cfg_set_tuple("outline", self.Conf.outline)
+            show_preview()
 
         def change_rounded(button):
             if button.get_active():
-                self.conf.corners = 0.14
+                self.Conf.corners = 0.14
             else:
-                self.conf.corners = 0
-            self.config_set("corners", str(self.conf.corners))
-            self.plugin_single_song(app.player.song)
+                self.Conf.corners = 0
+            self.config_set("corners", str(self.Conf.corners))
+            show_preview()
 
-        def edit_string(button):
-            w = PatternEdit(button, AnimOsd.conf.string)
-            w.text = self.conf.string
+        def edit_pattern(button):
+            w = PatternEdit(button, AnimOsd.ConfDef.string)
+            w.set_default_size(520, 260)
+            w.text = self.Conf.string
             w.apply.connect_object_after('clicked', set_string, w)
+            w.show()
 
         def set_string(window):
             value = window.text
             self.config_set("string", value)
-            self.conf.string = value
-            self.plugin_single_song(app.player.song)
+            self.Conf.string = value
+            show_preview()
 
         # Main VBox to return
         vb = Gtk.VBox(spacing=6)
 
-        # Display
-        vb2 = Gtk.VBox(spacing=3)
-        hb = Gtk.HBox(spacing=6)
-        # Set monitor to display OSD on if there's more than one
-        monitor_cnt = Gdk.Screen.get_default().get_n_monitors()
-        if monitor_cnt > 1:
-            adj = Gtk.Adjustment(value=self.conf.monitor, lower=0,
-                                 upper=monitor_cnt - 1, step_incr=1)
-            monitor = Gtk.SpinButton(adjustment=adj)
-            monitor.set_numeric(True)
-            monitor.connect('value-changed', change_monitor)
-            l2 = ConfigLabel("_Monitor:", monitor)
-            hb.pack_start(l2, False, True, 0)
-            hb.pack_start(monitor, False, True, 0)
-            vb2.pack_start(hb, True, True, 0)
-        else:
-            self.conf.monitor = 0 # should be this by default anyway
+        def build_display_widget():
+            vb2 = Gtk.VBox(spacing=3)
+            hb = Gtk.HBox(spacing=6)
+            # Set monitor to display OSD on if there's more than one
+            monitor_cnt = Gdk.Screen.get_default().get_n_monitors()
+            if monitor_cnt > 1:
+                adj = Gtk.Adjustment(value=self.Conf.monitor, lower=0,
+                                     upper=monitor_cnt - 1, step_incr=1)
+                monitor = Gtk.SpinButton(adjustment=adj)
+                monitor.set_numeric(True)
+                monitor.connect('value-changed', change_monitor)
+                l2 = ConfigLabel("_Monitor:", monitor)
+                hb.pack_start(l2, False, True, 0)
+                hb.pack_start(monitor, False, True, 0)
+                vb2.pack_start(hb, True, True, 0)
+            else:
+                # should be this by default anyway
+                self.Conf.monitor = 0
 
-        hb = Gtk.HBox(spacing=6)
-        cb = Gtk.ComboBoxText()
-        cb.append_text(_("Top of screen"))
-        cb.append_text(_("Middle of screen"))
-        cb.append_text(_("Bottom of screen"))
-        cb.set_active(int(self.conf.pos_y * 2.0))
-        cb.connect('changed', change_position)
-        lbl = ConfigLabel(_("_Position:"), cb)
+            hb = Gtk.HBox(spacing=6)
+            cb = Gtk.ComboBoxText()
+            cb.append_text(_("Top of screen"))
+            cb.append_text(_("Middle of screen"))
+            cb.append_text(_("Bottom of screen"))
+            cb.set_active(int(self.Conf.pos_y * 2.0))
+            cb.connect('changed', change_position)
+            lbl = ConfigLabel(_("_Position:"), cb)
 
-        hb.pack_start(lbl, False, True, 0)
-        hb.pack_start(cb, False, True, 0)
-        vb2.pack_start(hb, False, True, 0)
+            hb.pack_start(lbl, False, True, 0)
+            hb.pack_start(cb, False, True, 0)
+            vb2.pack_start(hb, False, True, 0)
+            return vb2
 
-        frame = qltk.Frame(label=_("Display"), child=vb2)
+        frame = qltk.Frame(label=_("Display"), child=build_display_widget())
         frame.set_border_width(6)
         vb.pack_start(frame, False, True, 0)
 
-        # Text
-        vb2 = Gtk.VBox(spacing=6)
-        hb = Gtk.HBox(spacing=6)
-        font = Gtk.FontButton()
-        font.set_font_name(self.conf.font)
-        font.connect('font-set', set_font)
-        lbl = ConfigLabel(_("_Font:"), font)
-        hb.pack_start(lbl, False, True, 0)
-        hb.pack_start(font, True, True, 0)
-        vb2.pack_start(hb, False, True, 0)
+        def build_text_widget():
+            t = Gtk.Table(2, 2)
+            t.set_col_spacings(6)
+            t.set_row_spacings(3)
 
-        hb = Gtk.HBox(spacing=3)
-        align = Gtk.ComboBoxText()
-        align.append_text(_("Left"))
-        align.append_text(_("Center"))
-        align.append_text(_("Right"))
-        align.set_active(self.conf.align)
-        align.connect('changed', change_align)
-        lbl = ConfigLabel(_("_Align text:"), align)
-        hb.pack_start(lbl, False, True, 0)
-        hb.pack_start(align, False, True, 0)
-        vb2.pack_start(hb, False, True, 0)
+            font = Gtk.FontButton(show_style=True)
+            font.set_font_name(self.Conf.font)
+            font.connect('font-set', set_font)
+            lbl = ConfigLabel(_("_Font:"), font)
+            t.attach(lbl, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL)
+            t.attach(font, 1, 2, 0, 1)
 
-        frame = qltk.Frame(label=_("Text"), child=vb2)
+            align = Gtk.ComboBoxText()
+            align.append_text(_("Left"))
+            align.append_text(_("Center"))
+            align.append_text(_("Right"))
+            align.set_active(self.Conf.align)
+            align.connect('changed', change_align)
+            lbl = ConfigLabel(_("_Align text:"), align)
+
+            t.attach(lbl, 0, 1, 1, 2, xoptions=Gtk.AttachOptions.FILL)
+            t.attach(align, 1, 2, 1, 2)
+            return t
+
+        frame = qltk.Frame(label=_("Text"), child=build_text_widget())
         frame.set_border_width(6)
         vb.pack_start(frame, False, True, 0)
 
-        # Colors
-        t = Gtk.Table(2, 2)
-        t.set_col_spacings(6)
-        t.set_row_spacings(3)
-        b = Gtk.ColorButton(rgba=Gdk.RGBA(*map(__floattocol, self.conf.text)))
-        l = Label(_("_Text:"))
-        l.set_mnemonic_widget(b)
-        l.set_use_underline(True)
-        t.attach(l, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL)
-        t.attach(b, 1, 2, 0, 1)
-        b.connect('color-set', set_text)
-        b = Gtk.ColorButton(color=Gdk.Color(*map(__floattocol,
-            self.conf.fill[0:3])))
-        b.set_use_alpha(True)
-        b.set_alpha(__floattocol(self.conf.fill[3]))
-        b.connect('color-set', set_fill)
-        l = Label(_("_Fill:"))
-        l.set_mnemonic_widget(b)
-        l.set_use_underline(True)
-        t.attach(l, 0, 1, 1, 2, xoptions=Gtk.AttachOptions.FILL)
-        t.attach(b, 1, 2, 1, 2)
+        def build_colors_widget():
+            t = Gtk.Table(2, 2)
+            t.set_col_spacings(6)
+            t.set_row_spacings(3)
+            b = Gtk.ColorButton(
+                rgba=Gdk.RGBA(*map(__floattocol, self.Conf.text)))
+            l = ConfigLabel(_("_Text:"), b)
 
-        f = qltk.Frame(label=_("Colors"), child=t)
+            t.attach(l, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL)
+            t.attach(b, 1, 2, 0, 1)
+            b.connect('color-set', set_text)
+            b = Gtk.ColorButton(color=Gdk.Color(*map(__floattocol,
+                                self.Conf.fill[0:3])))
+            b.set_use_alpha(True)
+            b.set_alpha(__floattocol(self.Conf.fill[3]))
+            b.connect('color-set', set_fill)
+            l = ConfigLabel(_("_Fill:"), b)
+            t.attach(l, 0, 1, 1, 2, xoptions=Gtk.AttachOptions.FILL)
+            t.attach(b, 1, 2, 1, 2)
+            return t
+
+        f = qltk.Frame(label=_("Colors"), child=build_colors_widget())
         f.set_border_width(6)
         vb.pack_start(f, False, False, 0)
 
-        # Effects
-        vb2 = Gtk.VBox(spacing=3)
-        hb = Gtk.HBox(spacing=6)
-        toggles = [
-            ("_Shadows", self.conf.shadow[0], change_shadow),
-            ("_Outline", self.conf.outline[0], change_outline),
-            ("Rou_nded Corners", self.conf.corners - 1, change_rounded),
-            ]
+        def build_effects_widget():
+            vb2 = Gtk.VBox(spacing=3)
+            hb = Gtk.HBox(spacing=6)
+            toggles = [
+                ("_Shadows", self.Conf.shadow[0], change_shadow),
+                ("_Outline", self.Conf.outline[0], change_outline),
+                ("Rou_nded Corners", self.Conf.corners - 1, change_rounded)]
 
-        for (label, current, callback) in toggles:
-            checkb = Gtk.CheckButton(label, use_underline=True)
-            checkb.set_active(current != -1)
-            checkb.connect("toggled", callback)
-            hb.pack_start(checkb, True, True, 0)
-        vb2.pack_start(hb, True, True, 0)
+            for (label, current, callback) in toggles:
+                checkb = Gtk.CheckButton(label, use_underline=True)
+                checkb.set_active(current != -1)
+                checkb.connect("toggled", callback)
+                hb.pack_start(checkb, True, True, 0)
+            vb2.pack_start(hb, True, True, 0)
 
-        hb = Gtk.HBox(spacing=6)
-        timeout = Gtk.SpinButton(adjustment=
-            Gtk.Adjustment(self.conf.delay / 1000.0, 0, 60, 0.1, 1.0, 0),
-            climb_rate=0.1, digits=1)
-        timeout.set_numeric(True)
-        timeout.connect('value-changed', change_delay)
-        l1 = ConfigLabel("_Delay:", timeout)
-        hb.pack_start(l1, False, True, 0)
-        hb.pack_start(timeout, False, True, 0)
-        vb2.pack_start(hb, False, True, 0)
+            hb = Gtk.HBox(spacing=6)
+            timeout = Gtk.SpinButton(
+                adjustment=Gtk.Adjustment(
+                    self.Conf.delay / 1000.0, 0, 60, 0.1, 1.0, 0),
+                climb_rate=0.1, digits=1)
+            timeout.set_numeric(True)
+            timeout.connect('value-changed', change_delay)
+            l1 = ConfigLabel("_Delay:", timeout)
+            hb.pack_start(l1, False, True, 0)
+            hb.pack_start(timeout, False, True, 0)
+            vb2.pack_start(hb, False, True, 0)
+            return vb2
 
-        frame = qltk.Frame(label=_("Effects"), child=vb2)
+        frame = qltk.Frame(label=_("Effects"), child=build_effects_widget())
         frame.set_border_width(6)
         vb.pack_start(frame, False, True, 0)
 
-        string = qltk.Button(_("Ed_it Display"), Gtk.STOCK_EDIT)
-        string.connect('clicked', edit_string)
-        vb.pack_start(string, False, True, 0)
+        def build_buttons_widget():
+            hb = Gtk.HBox(spacing=6)
+            edit_button = qltk.Button(_("Ed_it Display Pattern..."),
+                                      Gtk.STOCK_EDIT)
+            edit_button.connect('clicked', edit_pattern)
+            hb.pack_start(edit_button, False, True, 0)
+            preview_button = Gtk.Button(_("Preview"))
+            preview_button.connect("button-press-event", on_button_pressed)
+            hb.pack_start(preview_button, False, True, 0)
+            return hb
+
+        vb.pack_start(build_buttons_widget(), False, True, 0)
         return vb
 
-    class conf(object):
+    class ConfDef(object):
         # position of window 0--1 horizontal
         pos_x = 0.5
         # position of window 0--1 vertical
@@ -564,12 +594,14 @@ class AnimOsd(EventPlugin, PluginConfigMixin):
         # color,alpha or (-1.0,0.0,0.0,0.5) - borders the whole OSD
         bcolor = (0.0, 0.0, 0.0, 0.2)
         # song information to use - like in main window
-        string = (r"<album|\<b\><album>\</b\><discnumber| - Disc "
-r"""<discnumber>><part| - \<b\><part>\</b\>><tracknumber| - <tracknumber>>
->\<span weight='bold' size='large'\><title>\</span\> - <~length><version|
-\<small\>\<i\><version>\</i\>\</small\>><~people|"
+        string = (r"<album|[b]<album>[/b]<discnumber| - Disc "
+"""<discnumber>><part| - [b]<part>[/b]><tracknumber| - <tracknumber>>
+>[span weight='bold' size='large']<title>[/span] - <~length><version|
+[small][i]<version>[/i][/small]><~people|
 by <~people>>""")
 
+    Conf = ConfDef()
+
     def __init__(self):
         self.__current_window = None
 
@@ -595,7 +627,7 @@ by <~people>>""")
             ]
         for key, getconv in config_map:
             try:
-                default = getattr(self.conf, key)
+                default = getattr(self.Conf, key)
             except AttributeError:
                 print_d("Unknown config item '%s'" % key)
             try:
@@ -617,11 +649,7 @@ by <~people>>""")
                 if default is not None:
                     self.config_set(key, default)
             else:
-                setattr(self.conf, key, value)
-
-    # for rapid debugging and for the preview
-    def plugin_single_song(self, song):
-        self.plugin_on_song_started(song)
+                setattr(self.Conf, key, value)
 
     def plugin_on_song_started(self, song):
         if self.__current_window is not None:
@@ -635,7 +663,7 @@ by <~people>>""")
             self.__current_window = None
             return
 
-        window = OSDWindow(self.conf, song)
+        window = OSDWindow(self.Conf, song)
         window.add_events(Gdk.EventMask.BUTTON_PRESS_MASK)
         window.connect('button-press-event', self.__buttonpress)
         window.connect('fade-finished', self.__fade_finished)
@@ -645,7 +673,8 @@ by <~people>>""")
         window.show()
         window.fade_in()
 
-    def start_fade_out(self, window):
+    @staticmethod
+    def start_fade_out(window):
         window.fade_out()
         return False
 
@@ -657,7 +686,7 @@ by <~people>>""")
 
     def __fade_finished(self, window, fade_in):
         if fade_in:
-            GLib.timeout_add(self.conf.delay, self.start_fade_out, window)
+            GLib.timeout_add(self.Conf.delay, self.start_fade_out, window)
         else:
             window.hide()
             if self.__current_window is window:
