@@ -1,9 +1,15 @@
-# Copyright 2011 Christoph Reiter <christoph.reiter@gmx.at>
+# Copyright 2011 Christoph Reiter <reiter.christoph@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of version 2 of the GNU General Public License as
 # published by the Free Software Foundation.
 
+import os
+
+if os.name == "nt":
+    from quodlibet.plugins import PluginNotSupportedError
+    raise PluginNotSupportedError
+
 from gi.repository import Gtk
 import dbus
 
