@@ -1,9 +1,15 @@
-# Copyright 2010,2012 Christoph Reiter <christoph.reiter@gmx.at>
+# Copyright 2010,2012 Christoph Reiter <reiter.christoph@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of version 2 of the GNU General Public License as
 # published by the Free Software Foundation.
 
+import os
+
+if os.name == "nt":
+    from quodlibet.plugins import PluginNotSupportedError
+    raise PluginNotSupportedError
+
 import time
 import tempfile
 
@@ -57,8 +63,13 @@ class MPRIS(EventPlugin):
     def enabled(self):
         self.__sig = app.window.connect('delete-event', self.__window_delete)
 
-        self.objects = [MPRIS1Root(), MPRIS1DummyTracklist(),
-                        MPRIS1Player(), MPRIS2()]
+        self.objects = []
+        for service in [MPRIS1Root, MPRIS1DummyTracklist,
+                        MPRIS1Player, MPRIS2]:
+            try:
+                self.objects.append(service())
+            except dbus.DBusException:
+                pass
 
         # Needed for sound menu support in some older Ubuntu versions
         if indicate:
