@@ -1,4 +1,4 @@
-# Copyright 2011-2012 Nick Boultbee
+# Copyright 2011-2014 Nick Boultbee
 #
 # Inspired in parts by PySqueezeCenter (c) 2010 JingleManSweep
 # SqueezeCenter and SqueezeBox are copyright Logitech
@@ -6,9 +6,13 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
 
-import os.path
+import os
+
+if os.name == "nt":
+    from quodlibet.plugins import PluginNotSupportedError
+    raise PluginNotSupportedError
+
 from telnetlib import Telnet
 from threading import Thread
 import socket
@@ -22,11 +26,13 @@ from quodlibet import config
 from quodlibet import qltk
 from quodlibet.plugins.events import EventPlugin
 from quodlibet.plugins import PluginConfigMixin
-from quodlibet.plugins.songsmenu import SongsMenuPlugin
+from quodlibet.plugins.playlist import PlaylistPlugin
 from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.msg import Message
 from quodlibet.qltk.notif import Task
 from quodlibet.util import copool
+from quodlibet.util.dprint import print_d, print_w, print_
+from quodlibet.util.library import get_scan_dirs
 
 
 class SqueezeboxServerSettings(dict):
@@ -64,7 +70,7 @@ class SqueezeboxServer(object):
     _debug = False
 
     def __init__(self, hostname="localhost", port=9090, user="", password="",
-        library_dir='', current_player=0, debug=False):
+                 library_dir='', current_player=0, debug=False):
         self._debug = debug
         self.failures = 0
         self.delta = 600    # Default in ms
@@ -84,7 +90,7 @@ class SqueezeboxServer(object):
                 if result != (6 * '*'):
                     raise SqueezeboxException(
                         "Couldn't log in to squeezebox: response was '%s'"
-                         % result)
+                        % result)
                 self.is_connected = True
                 self.failures = 0
                 print_d("Connected to Squeezebox Server! %s" % self)
@@ -112,7 +118,7 @@ class SqueezeboxServer(object):
             if not want_reply:
                 return None
             raw_response = self.telnet.read_until("\n").strip()
-        except socket.error, e:
+        except socket.error as e:
             print_w("Couldn't communicate with squeezebox (%s)" % e)
             self.failures += 1
             if self.failures >= self._MAX_FAILURES:
@@ -194,9 +200,9 @@ class SqueezeboxServer(object):
         self.player_request("playlist clear", False)
 
     def playlist_resume(self, name, resume, wipe=False):
-        self.player_request("playlist resume %s noplay:%d wipePlaylist:%d" %
-                            (urllib.quote(name), int(not resume), int(wipe)),
-                           want_reply=False)
+        cmd = ("playlist resume %s noplay:%d wipePlaylist:%d"
+               % (urllib.quote(name), int(not resume), int(wipe)))
+        self.player_request(cmd, want_reply=False)
 
     def change_song(self, path):
         """Queue up a song"""
@@ -252,7 +258,7 @@ class GetPlayerDialog(Gtk.Dialog):
         title = _("Choose Squeezebox player")
         super(GetPlayerDialog, self).__init__(title, parent)
         self.set_border_width(6)
-        self.set_has_separator(False)
+        #self.set_has_separator(False)
         self.set_resizable(False)
         self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                          Gtk.STOCK_OK, Gtk.ResponseType.OK)
@@ -296,13 +302,15 @@ class SqueezeboxPluginMixin(PluginConfigMixin):
 
     # Maintain a singleton; we only support one SB server live in QL
     server = None
-    ql_base_dir = os.path.realpath(config.get("settings", "scan"))
+    ql_base_dir = (os.path.realpath(get_scan_dirs()[0])
+                   if get_scan_dirs() else "")
+    print_d("Using QL library dir of %s" % ql_base_dir)
 
     # We want all derived classes to share the config section
     CONFIG_SECTION = "squeezebox"
 
     @classmethod
-    def get_path(cls, song):
+    def get_sb_path(cls, song):
         """Gets a SB path to `song` by simple substitution"""
         path = song('~filename')
         return path.replace(cls.ql_base_dir, cls.server.get_library_dir())
@@ -338,7 +346,7 @@ class SqueezeboxPluginMixin(PluginConfigMixin):
                 ret = dialog.run() or 0
             else:
                 cls.quick_dialog("Squeezebox OK. Using the only player (%s)."
-                                  % cls.server.players[0])
+                                 % cls.server.players[0])
             cls.set_player(ret)
             # TODO: verify sanity of SB library path
 
@@ -347,7 +355,7 @@ class SqueezeboxPluginMixin(PluginConfigMixin):
 
         else:
             cls.quick_dialog(_("Couldn't connect to %s") % (cls.server,),
-                              Gtk.MessageType.ERROR)
+                             Gtk.MessageType.ERROR)
 
     @classmethod
     def PluginPreferences(cls, parent):
@@ -441,7 +449,7 @@ class SqueezeboxPluginMixin(PluginConfigMixin):
                 print_d(
                     "Squeezebox server version: %s. Current player: #%d (%s)."
                     % (ver, cur, cls.server.get_players()[cur]["name"]))
-        except (IndexError, KeyError), e:
+        except (IndexError, KeyError, SqueezeboxException) as e:
             print_d("Couldn't get player info (%s)." % e)
 
 
@@ -449,7 +457,7 @@ class SqueezeboxSyncPlugin(EventPlugin, SqueezeboxPluginMixin):
     PLUGIN_ID = 'Squeezebox Output'
     PLUGIN_NAME = _('Squeezebox Sync')
     PLUGIN_DESC = _("Make Logitech Squeezebox mirror Quod Libet output, "
-            "provided both read from an identical library")
+                    "provided both read from an identical library")
     PLUGIN_ICON = Gtk.STOCK_MEDIA_PLAY
     PLUGIN_VERSION = '0.3'
     server = None
@@ -494,7 +502,7 @@ class SqueezeboxSyncPlugin(EventPlugin, SqueezeboxPluginMixin):
         if cls._debug:
             print_d("Paused" if app.player.paused else "Not paused")
         if song and cls.server and cls.server.is_connected:
-            path = cls.get_path(song)
+            path = cls.get_sb_path(song)
             print_d("Requesting to play %s..." % path)
             if app.player.paused:
                 cls.server.change_song(path)
@@ -521,14 +529,14 @@ class SqueezeboxSyncPlugin(EventPlugin, SqueezeboxPluginMixin):
             pass
 
 
-class SqueezeboxPlaylistPlugin(SongsMenuPlugin, SqueezeboxPluginMixin):
+class SqueezeboxPlaylistPlugin(PlaylistPlugin, SqueezeboxPluginMixin):
     PLUGIN_ID = "Export to Squeezebox Playlist"
-    PLUGIN_NAME = _("Export to Squeezebox Playlist")
-    PLUGIN_DESC = _("Dynamically export songs to Logitech Squeezebox "
-                    "playlists, provided both share a directory structure. "
+    PLUGIN_NAME = _("Export to Squeezebox...")
+    PLUGIN_DESC = _("Dynamically export a playlist to Logitech Squeezebox "
+                    "playlist, provided both share a directory structure. "
                     "Shares configuration with Squeezebox Sync plugin")
-    PLUGIN_ICON = Gtk.STOCK_EDIT
-    PLUGIN_VERSION = '0.2'
+    PLUGIN_ICON = Gtk.STOCK_CONNECT
+    PLUGIN_VERSION = '0.4'
     TEMP_PLAYLIST = "_quodlibet"
 
     def __add_songs(self, task, songs, name):
@@ -549,7 +557,7 @@ class SqueezeboxPlaylistPlugin(SongsMenuPlugin, SqueezeboxPluginMixin):
                 break
             # Actually do the (slow) call
             worker = Thread(target=self.server.playlist_add,
-                            args=(self.get_path(song),))
+                            args=(self.get_sb_path(song),))
             worker.daemon = True
             worker.start()
             worker.join(timeout=3)
@@ -569,15 +577,16 @@ class SqueezeboxPlaylistPlugin(SongsMenuPlugin, SqueezeboxPluginMixin):
         """Tell the copool to stop (adding songs)"""
         self.__cancel = True
 
-    def __get_playlist_name(self):
+    @staticmethod
+    def __get_playlist_name(name="Quod Libet playlist"):
         dialog = qltk.GetStringDialog(None,
-            _("Export selection to Squeezebox playlist"),
+            _("Export playlist to Squeezebox"),
             _("Playlist name (will overwrite existing)"),
             okbutton=Gtk.STOCK_SAVE)
-        name = dialog.run(text="Quod Libet playlist")
+        name = dialog.run(text=name)
         return name
 
-    def plugin_songs(self, songs):
+    def plugin_playlist(self, playlist):
         self.init_server()
         if not self.server.is_connected:
             qltk.ErrorMessage(
@@ -587,8 +596,9 @@ class SqueezeboxPlaylistPlugin(SongsMenuPlugin, SqueezeboxPluginMixin):
                 self.server.config
             ).run()
         else:
-            name = self.__get_playlist_name()
-            task = Task("Squeezebox", _("Export to Squeezebox playlist"),
-                        stop=self.__cancel_add)
-            copool.add(self.__add_songs, task, songs, name,
-                       funcid="squeezebox-playlist-save")
+            name = self.__get_playlist_name(name=playlist.name)
+            if name:
+                task = Task("Squeezebox", _("Export to Squeezebox playlist"),
+                            stop=self.__cancel_add)
+                copool.add(self.__add_songs, task, playlist.songs, name,
+                           funcid="squeezebox-playlist-save")
