@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2006 Joe Wreschnig, Michael Urman, Iñigo Serna
 #           2012 Christoph Reiter
+#           2013 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -11,10 +12,10 @@ import sys
 from gi.repository import Gtk, Pango, Gdk, GdkPixbuf, GLib
 
 from quodlibet import browsers, config, qltk, util, app
+from quodlibet.config import RATINGS
 from quodlibet.parse import Pattern
 from quodlibet.plugins.events import EventPlugin
 from quodlibet.qltk.browser import LibraryBrowser
-from quodlibet.qltk.controls import StopAfterMenu
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.playorder import ORDERS
 from quodlibet.qltk.properties import SongProperties
@@ -81,7 +82,8 @@ class Preferences(Gtk.VBox):
                 "<album|<album~discnumber~part~tracknumber~title~version>|"
                 "<artist~title~version>>")
 
-        self.show_all()
+        for child in self.get_children():
+            child.show_all()
 
     def __changed_combo(self, combo):
         config.set(
@@ -120,7 +122,7 @@ class Preferences(Gtk.VBox):
 
 
 class TrayIcon(EventPlugin):
-    __icon = None
+    _icon = None
     __pixbuf = None
     __pixbuf_paused = None
     __icon_theme = None
@@ -129,7 +131,6 @@ class TrayIcon(EventPlugin):
     __w_sig_map = None
     __w_sig_del = None
     __theme_sig = None
-    __stop_after = None
     __first_map = True
     __pattern = Pattern(
         "<album|<album~discnumber~part~tracknumber~title~version>|"
@@ -141,43 +142,40 @@ class TrayIcon(EventPlugin):
     PLUGIN_VERSION = "2.0"
 
     def enabled(self):
-        self.__icon = Gtk.StatusIcon()
+        self._icon = Gtk.StatusIcon()
         self.__icon_theme = Gtk.IconTheme.get_default()
         self.__theme_sig = self.__icon_theme.connect('changed',
             self.__theme_changed)
 
-        self.__icon.connect('size-changed', self.__size_changed)
-        #no size-changed under win32
+        self._icon.connect('size-changed', self.__size_changed)
+        # No size-changed under win32
         if sys.platform == "win32":
             self.__size = 16
 
-        self.__icon.connect('popup-menu', self.__button_right)
-        self.__icon.connect('activate', self.__button_left)
+        self._icon.connect('popup-menu', self._popup_menu)
+        self._icon.connect('activate', self.__button_left)
 
-        self.__icon.connect('scroll-event', self.__scroll)
-        self.__icon.connect('button-press-event', self.__button_middle)
+        self._icon.connect('scroll-event', self.__scroll)
+        self._icon.connect('button-press-event', self.__button_middle)
 
-        self.__w_sig_map = app.window.connect('map', self.__window_map)
+        self.__w_sig_map = app.window.connect('map-event', self.__window_map)
         self.__w_sig_del = app.window.connect('delete-event',
                                               self.__window_delete)
 
-        self.__stop_after = StopAfterMenu(app.player)
-
         self.plugin_on_paused()
         self.plugin_on_song_started(app.player.song)
 
     def disabled(self):
         self.__icon_theme.disconnect(self.__theme_sig)
         self.__icon_theme = None
-        self.__stop_after = None
         app.window.disconnect(self.__w_sig_map)
         app.window.disconnect(self.__w_sig_del)
-        self.__icon.set_visible(False)
+        self._icon.set_visible(False)
         try:
-            self.__icon.destroy()
+            self._icon.destroy()
         except AttributeError:
             pass
-        self.__icon = None
+        self._icon = None
         self.__show_window()
 
     def PluginPreferences(self, parent):
@@ -186,7 +184,7 @@ class TrayIcon(EventPlugin):
         return p
 
     def __get_paused_pixbuf(self, size, diff):
-        """Returns a pixbuf for a paused icon frokm the current theme.
+        """Returns a pixbuf for a paused icon from the current theme.
         The returned pixbuf can have a size of size->size+diff"""
 
         names = ('media-playback-pause', Gtk.STOCK_MEDIA_PAUSE)
@@ -219,11 +217,11 @@ class TrayIcon(EventPlugin):
                 util.print_exc()
                 return
 
-        #we need to fill the whole height that is given to us, or
-        #the KDE panel will emit size-changed until we reach 0
+        # We need to fill the whole height that is given to us, or
+        # the KDE panel will emit size-changed until we reach 0
         w, h = self.__pixbuf.get_width(), self.__pixbuf.get_height()
         if h < self.__size:
-            bg = GdkPixbuf.Pixbuf(
+            bg = GdkPixbuf.Pixbuf.new(
                 GdkPixbuf.Colorspace.RGB, True, 8, w, self.__size)
             bg.fill(0)
             self.__pixbuf.copy_area(0, 0, w, h, bg, 0, (self.__size - h) / 2)
@@ -245,9 +243,9 @@ class TrayIcon(EventPlugin):
                 wo, ho = overlay.get_width(), overlay.get_height()
 
                 overlay.composite(base, w - wo - pad, h - ho - pad,
-                    wo, ho, w - wo - pad, h - ho - pad,
-                    1, 1,
-                    GdkPixbuf.InterpType.BILINEAR, 255)
+                                  wo, ho, w - wo - pad, h - ho - pad,
+                                  1, 1,
+                                  GdkPixbuf.InterpType.BILINEAR, 255)
 
             self.__pixbuf_paused = base
 
@@ -256,7 +254,7 @@ class TrayIcon(EventPlugin):
         else:
             new_pixbuf = self.__pixbuf
 
-        self.__icon.set_from_pixbuf(new_pixbuf)
+        self._icon.set_from_pixbuf(new_pixbuf)
 
     def __theme_changed(self, theme, *args):
         self.__pixbuf = None
@@ -273,18 +271,18 @@ class TrayIcon(EventPlugin):
         return True
 
     def __prefs_destroy(self, *args):
-        if self.__icon:
+        if self._icon:
             self.plugin_on_song_started(app.player.song)
 
     def __window_delete(self, win, event):
         return self.__hide_window()
 
-    def __window_map(self, win):
+    def __window_map(self, win, *args):
         visible = config.getboolean("plugins", "icon_window_visible", False)
 
         config.set("plugins", "icon_window_visible", "true")
 
-        #only restore window state on start
+        # Only restore window state on start
         if not visible and self.__first_map:
             self.__hide_window()
 
@@ -306,7 +304,8 @@ class TrayIcon(EventPlugin):
             self.__show_window()
 
     def __button_middle(self, widget, event):
-        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 2:
+        if event.type == Gdk.EventType.BUTTON_PRESS and \
+                event.button == Gdk.BUTTON_MIDDLE:
             if self.__destroy_win32_menu():
                 return
             self.__play_pause()
@@ -342,7 +341,7 @@ class TrayIcon(EventPlugin):
                 player.volume -= 0.05
 
     def plugin_on_song_started(self, song):
-        if not self.__icon:
+        if not self._icon:
             return
 
         if song:
@@ -355,7 +354,7 @@ class TrayIcon(EventPlugin):
         else:
             tooltip = _("Not playing")
 
-        self.__icon.set_tooltip_markup(util.escape(tooltip))
+        self._icon.set_tooltip_markup(util.escape(tooltip))
 
     def __destroy_win32_menu(self):
         """Returns True if current action should only hide the menu"""
@@ -364,7 +363,7 @@ class TrayIcon(EventPlugin):
             self.__menu = None
             return True
 
-    def __button_right(self, icon, button, time):
+    def _popup_menu(self, icon, button, time):
         if self.__destroy_win32_menu():
             return
         self.__menu = menu = Gtk.Menu()
@@ -389,13 +388,14 @@ class TrayIcon(EventPlugin):
         repeat.connect('toggled',
             lambda s: window.repeat.set_active(s.get_active()))
 
-        def set_safter(widget, stop_after):
-            stop_after.active = widget.get_active()
+        def set_safter(widget, safter_action):
+            safter_action.set_active(widget.get_active())
 
+        safter_action = app.window.stop_after
         safter = Gtk.CheckMenuItem(label=_("Stop _after this song"),
                                    use_underline=True)
-        safter.set_active(self.__stop_after.active)
-        safter.connect('activate', set_safter, self.__stop_after)
+        safter.set_active(safter_action.get_active())
+        safter.connect('toggled', set_safter, safter_action)
 
         def set_order(widget, num):
             window.order.set_active(num)
@@ -416,7 +416,8 @@ class TrayIcon(EventPlugin):
         order_sub.append(repeat)
         order_sub.append(safter)
         order_sub.append(SeparatorMenuItem())
-        map(order_sub.append, order_items)
+        for item in order_items:
+            order_sub.append(item)
         orders.set_submenu(order_sub)
 
         browse = qltk.MenuItem(_("_Browse Library"), Gtk.STOCK_FIND)
@@ -426,7 +427,8 @@ class TrayIcon(EventPlugin):
             if not Kind.in_menu:
                 continue
             i = Gtk.MenuItem(label=Kind.accelerated_name, use_underline=True)
-            i.connect_object('activate', LibraryBrowser, Kind, app.library)
+            i.connect_object(
+                'activate', LibraryBrowser.open, Kind, app.library)
             browse_sub.append(i)
 
         browse.set_submenu(browse_sub)
@@ -437,9 +439,6 @@ class TrayIcon(EventPlugin):
         info = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_INFO, None)
         info.connect('activate', self.__information)
 
-        rating = Gtk.MenuItem(label=_("_Rating"), use_underline=True)
-        rating_sub = Gtk.Menu()
-
         def set_rating(value):
             song = player.song
             if song is None:
@@ -448,12 +447,12 @@ class TrayIcon(EventPlugin):
                 song["~#rating"] = value
                 app.librarian.changed([song])
 
-        for i in range(0, int(1.0 / util.RATING_PRECISION) + 1):
-            j = i * util.RATING_PRECISION
-            item = Gtk.MenuItem("%0.2f\t%s" % (j, util.format_rating(j)))
-            item.connect_object('activate', set_rating, j)
+        rating = Gtk.MenuItem(label=_("_Rating"), use_underline=True)
+        rating_sub = Gtk.Menu()
+        for r in RATINGS.all:
+            item = Gtk.MenuItem("%0.2f\t%s" % (r, util.format_rating(r)))
+            item.connect_object('activate', set_rating, r)
             rating_sub.append(item)
-
         rating.set_submenu(rating_sub)
 
         quit = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_QUIT, None)
@@ -475,11 +474,13 @@ class TrayIcon(EventPlugin):
 
         menu.show_all()
 
-        if sys.platform == "win32":
-            menu.popup(None, None, None, button, time, self.__icon)
+        if sys.platform != "win32":
+            pos_func = Gtk.StatusIcon.position_menu
+            pos_arg = self._icon
         else:
-            menu.popup(None, None, Gtk.StatusIcon.position_menu, self.__icon,
-                button, time)
+            pos_func = pos_arg = None
+
+        menu.popup(None, None, pos_func, pos_arg, button, time)
 
     plugin_on_paused = __update_icon
     plugin_on_unpaused = __update_icon
@@ -487,9 +488,11 @@ class TrayIcon(EventPlugin):
     def __properties(self, *args):
         song = app.player.song
         if song:
-            SongProperties(app.librarian, [song])
+            window = SongProperties(app.librarian, [song])
+            window.show()
 
     def __information(self, *args):
         song = app.player.song
         if song:
-            Information(app.librarian, [song])
+            window = Information(app.librarian, [song])
+            window.show()
