@@ -7,8 +7,8 @@
 
 from quodlibet.plugins.playorder import PlayOrderPlugin, PlayOrderInOrderMixin
 
-# Importing main directly fails, as it is assigned after this file is read
-from quodlibet import widgets
+from quodlibet import app
+
 
 class QueueOrder(PlayOrderPlugin, PlayOrderInOrderMixin):
     PLUGIN_ID = "queue"
@@ -22,8 +22,9 @@ class QueueOrder(PlayOrderPlugin, PlayOrderInOrderMixin):
         return None
 
     def set_explicit(self, playlist, iter):
-        if iter is None: return
+        if iter is None:
+            return
         song = playlist[iter][0]
-        if song is None: return
-        widgets.main.playlist.enqueue([playlist[iter][0]])
-
+        if song is None:
+            return
+        app.window.playlist.enqueue([playlist[iter][0]])
