@@ -17,12 +17,10 @@ from gi.repository import Gtk, GObject, Pango, GLib
 try:
     from musicbrainz2 import webservice as ws
     from musicbrainz2.utils import extractUuid
-except ImportError:
+except ImportError as e:
     from quodlibet import plugins
-    if not hasattr(plugins, "PluginImportException"):
-        raise
-    raise plugins.PluginImportException(
-        "Couldn't find python-musicbrainz2.")
+    raise (plugins.MissingModulePluginException("musicbrainz2") if
+           hasattr(plugins, "MissingModulePluginException") else e)
 
 from quodlibet import config, util
 from quodlibet.qltk.ccb import ConfigCheckButton
@@ -37,7 +35,8 @@ def get_artist(album):
     for tag in ["albumartist", "artist", "performer"]:
         names = set()
         for song in album:
-            map(names.add, filter(lambda n: n, song.get(tag, "").split("\n")))
+            for single in filter(None, song.get(tag, "").split("\n")):
+                names.add(single)
         if len(names) == 1:
             return names.pop()
         elif len(names) > 1:
@@ -47,8 +46,20 @@ def get_artist(album):
 
 def get_trackcount(album):
     """Returns the track count, hammered into submission."""
-    return max(max(map(lambda t: max(map(int,
-        t.get('tracknumber', '0').split('/'))), album)), len(album)) # (;))
+
+    parts = []
+    for song in album:
+        parts.extend(song.get("tracknumber", "0").split("/"))
+
+    max_count = len(album)
+    for part in parts:
+        try:
+            tracks = int(part)
+        except ValueError:
+            continue
+        max_count = max(max_count, tracks)
+
+    return max_count
 
 
 def config_get(key, default=''):
@@ -97,7 +108,8 @@ class ResultTreeView(HintedTreeView, MultiDragTreeView):
         self.album = album
         self.remote_album = []
         self.model = Gtk.ListStore(object)
-        map(self.model.append, zip(album))
+        for song in album:
+            self.model.append([song])
 
         super(ResultTreeView, self).__init__(self.model)
         self.set_headers_clickable(True)
@@ -214,7 +226,7 @@ class ReleaseEventComboBox(Gtk.HBox):
             return None
 
 
-class QueryThread:
+class QueryThread(object):
     """Daemon thread which does HTTP retries and avoids flooding."""
     def __init__(self):
         self.running = True
@@ -244,7 +256,11 @@ class QueryThread:
                         res = func(*args, **kwargs)
                     except:
                         res = None
-                GLib.idle_add(callback, res)
+
+                def idle_check(cb, res):
+                    if self.running:
+                        cb(res)
+                GLib.idle_add(idle_check, callback, res)
             time.sleep(1)
 
 
@@ -386,7 +402,7 @@ class SearchWindow(Gtk.Dialog):
         save_button = dialog_get_widget_for_stockid(self, Gtk.STOCK_SAVE)
         save_button.set_sensitive(True)
 
-    def __init__(self, album, cache):
+    def __init__(self, parent, album, cache):
         self.album = album
 
         self._query = ws.Query()
@@ -400,6 +416,7 @@ class SearchWindow(Gtk.Dialog):
                     Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
         self.set_default_size(650, 500)
         self.set_border_width(5)
+        self.set_transient_for(parent)
 
         save_button = dialog_get_widget_for_stockid(self, Gtk.STOCK_SAVE)
         save_button.set_sensitive(False)
@@ -458,7 +475,7 @@ class SearchWindow(Gtk.Dialog):
         self.connect('response', self.__save)
 
         stb.emit('clicked')
-        self.show_all()
+        self.get_child().show_all()
 
 
 class MyBrainz(SongsMenuPlugin):
@@ -471,13 +488,31 @@ class MyBrainz(SongsMenuPlugin):
     cache = {}
 
     def plugin_albums(self, albums):
+        discs_todo = []
         for album in albums:
             discs = {}
             for song in album:
                 discnum = int(song.get('discnumber', '1').split('/')[0])
                 discs.setdefault(discnum, []).append(song)
             for disc in discs.values():
-                SearchWindow(disc, self.cache).run()
+                discs_todo.append(disc)
+
+        if not discs_todo:
+            return
+
+        def win_finished_cb(widget, *args):
+            if discs_todo:
+                start_processing(discs_todo.pop(0))
+            else:
+                self.plugin_finish()
+
+        def start_processing(disc):
+            win = SearchWindow(
+                self.plugin_window, disc, self.cache)
+            win.connect("destroy", win_finished_cb)
+            win.show()
+
+        start_processing(discs_todo.pop(0))
 
     @classmethod
     def PluginPreferences(self, win):
